#! /bin/sh
#
# Script to install the new dbengine
# database-management-system
#
# You can call the script with 5 parameters:
#    1. Option -c: the directory of the cgi-scripts (default: cgi-bin)
#    2. Option -d: the directory of the other parts of the system
#       (default: /usr/local/dbengine)
#    3. Option -n: name of the server for the postgreSQL
#       databases (default: localhost)
#    4. Option -p: port of the server for the postgreSQL
#       databases (default: 5432)
#    5. Option -a: name of postgreSQL-admin, i.e. the main user
#       for the dbengine_info_db database
#       (default: postgres)
#    example with the default parameters:
#       install.sh -c cgi-bin -d /usr/local/dbengine -n localhost -p 5432 -a postgres
#
# Version 2.3.0
# <c> 2003 Detlef Duerr
# get closer Information from email Detlef.Duerr@gmx.de
#
set -x
#

function createdir () {
   DIRN=$1
   if [ -x "$DIRN" ]
   then
#      RETURN=0
      return 0
   fi
   echo -n "The directory $DIRN does not exist, is it to create [yn]? "
   read YESNO
   if [ "x$YESNO" = xy -o "x$YESNO" = xY ]
   then
      mkdir -p $DIRN
      if [ $? -ne 0 ]
      then
         echo "+++ Error: couldn't create directory $DIRN"
         exit 1
      fi
#      RETURN=0
      return 0
   fi
#   RETURN=1
   return 1
}

while [ "x$1" != x ]
do
   paramk=$1
   shift
   if [ "x$1" = x ]
   then
      echo "+++ Error: Value for parameter $paramk missing"
      exit 1
   fi
   paramv=$1
   shift
   case $paramk
   in
      "-c")
         CGIDIR=$paramv
         ;;
      "-d")
         DBENGDIR=$paramv
         ;;
      "-n")
         SERVER=$paramv
         ;;
      "-p")
         PORT=$paramv
         ;;
      "-a")
         ADMIN=$paramv
         ;;
       "*")
         echo "+++ Error: illegal parameter $paramk"
         ;;
   esac
done

if [ -z $CGIDIR ]
then
   echo -n "The directory of the cgi-scripts [/usr/local/httpd/cgi-bin]: "
   read CGIDIR
   if [ "x$CGIDIR" = x ]
   then
      CGIDIR=/usr/local/httpd/cgi-bin
   fi
fi
if [ -z $DBENGDIR ]
then
   echo -n "The directory of the dbengine-system [/usr/local/dbengine]: "
   read DBENGDIR
   if [ "x$DBENGDIR" = x ]
   then
      DBENGDIR=/usr/local/dbengine
   fi
fi
if [ -z $SERVER ]
then
   echo -n "The server name [localhost]: "
   read SERVER
   if [ "x$SERVER" = x ]
   then
      SERVER="localhost"
   fi
fi
if [ -z $PORT ]
then
   echo -n "The server port [5432]: "
   read PORT
   if [ "x$PORT" = x ]
   then
      PORT="5432"
   fi
fi
if [ -z $ADMIN ]
then
   echo -n "The postgreSQL ADMIN [postgres]: "
   read ADMIN
   if [ "x$ADMIN" = x ]
   then
      ADMIN="postgres"
   fi
fi

echo "You have defined the following system-data for installing the dbengine:"
echo "     the cgi-directory:          $CGIDIR"
echo "     the dbengine-directory:     $DBENGDIR"
echo "     the postgreSQL-server name: $SERVER"
echo "     the postgreSQL-server port: $PORT"
echo "     the postgreSQL ADMIN:        $ADMIN"
while [ 1 ]
do
   echo -n "This definitions ok, starting the installation [yN]: "
   read YESNO
   if [ "x$YESNO" = xy -o "x$YESNO" = xY ]
   then
      YESNO=OK
      break
   elif [ "x$YESNO" = xn -o "x$YESNO" = xN ]
   then
      exit 1
   fi
done
#
# starting the installation
# you must be root to start the installation
#
if [ `id -u` -ne 0 ]
then
   echo "You must be root to start the installation"
   exit 1
fi

OLDDIR=`pwd`

createdir $CGIDIR
if [ $? -ne 0 ]
then
   exit 1
fi

createdir $DBENGDIR
if [ $? -ne 0 ]
then
   exit 1
fi

#
# creating the cgi-script in the proper location
#
cat >$CGIDIR/dbengine.cgi <<EOM
#!/usr/bin/perl -w
#
# dbengine.cgi  Version 2.3.0
# <c> 1997, 1998, 1999, 2003, 2004 Detlef Duerr
# get closer Information from email Detlef.Duerr@gmx.de


#
# and here are the definition for the directory
# where we can find all the other modules.
# This we have to set for running
# an actual databasemanagement-system
#
EOM

echo "my \$refdbengine = \"$DBENGDIR/src\";" >>$CGIDIR/dbengine.cgi
echo "my \$server      = \"$SERVER\";" >>$CGIDIR/dbengine.cgi
echo "my \$port        = \"$PORT\";" >>$CGIDIR/dbengine.cgi

cat >>$CGIDIR/dbengine.cgi <<EOM
require "\$refdbengine/dbengine.pl";

&start_dbengine(\$refdbengine,\$server,\$port);

exit(0);
EOM

#
# copying all files to the dbengine-directory
#
chmod 555 $CGIDIR/dbengine.cgi

cp -a . $DBENGDIR

#
# creating the standard postgreSQL-database if neccessary
#
DBASEN=`psql -h $SERVER -p $PORT -U $ADMIN -l | grep dbengine_info_db`
if [ "x$DBASEN" = x ]
then
   createdb -h $SERVER -p $PORT -U $ADMIN dbengine_info_db
   psql -h $SERVER -p $PORT -U $ADMIN dbengine_info_db <dbengine_info_db.sql
fi
#
# if the ADMIN isn't included as admin in the database include the new admin
#
ADMINS="SELECT values FROM configs WHERE index = 'admins'"
ADMIN_OLD=`psql -h $SERVER -p $PORT -U $ADMIN -c $ADMINS -t -A dbengine_info_db`
if [ "x$ADMIN_OLD" = x  -o "x$ADMIN_OLD" != "$ADMIN" ]
then
   ADMINS="INSERT INTO configs (index, values, comment, tmin)"
   ADMINS="$ADMINS VALUES ('admins','$ADMIN','List of usernames of people working as dbengine-admins separated by commas','now')"
   psql -h $SERVER -p $PORT -U $ADMIN -c $ADMINS dbengine_info_db
fi
#
# loading the test-database with data if necessary
#
DBASEN=`psql -h $SERVER -p $PORT -U $ADMIN -l | grep dbengine_test_db`
if [ "x$DBASEN" = x ]
then
   createdb -h $SERVER -p $PORT -U $ADMIN dbengine_test_db
   psql -h $SERVER -p $PORT -U $ADMIN dbengine_test_db <test/dbengine_test_db.sql
fi

