#########################
#
# dbengine.pl
#
# Modul zu dbengine.cgi: Verwaltung und Nutzung von Datenbanken ber
# Browserschnittstelle.
#
# Version 2.3.2
# <c> 2003, 2004 Detlef Duerr
# get closer Information from email Detlef.Duerr@gmx.de
# Version 1.01
# <c> 1997, 1998, 1999 Detlef Duerr
# get closer Information from email Detlef.Duerr@atm-computer.de
# older version
# <c> 1996 Ingo Ciechowski
# see closer information at http://www.cis-computer.com/dbengine
#
# Erforderliche Resourcen:
# 1. Module for Perl:
#       Mcrypt
#       MIME::Base64
#       DBI ( actually tested for DBD::Pg version 1.21)
#       DBD::Pg
#       CGI
#
########################

use strict;

my $debug      = 0;     # enable output of some debugging infos
my $debug_co   = 1;     # debug outputs as comments
my $debug_file = 0;     # debug outputs in a file

#
# Initialisierungen fr das System
#
use Crypt::CBC;
use MIME::Base64;
use CGI;
use DBI;

#
# Generelle Variable
#
my %configs    = ("server"     => "localhost",        # domain of the server where our database lives
                  "port"       => "5432",               # port of the database server
                  'descdb'     => "dbi:Pg:dbname=dbengine_info_db",
                  "descdbn"    => "dbengine_info_db", # name of default database with formating options
                  "dbsystem"   => "Pg",               # DBD - Database driver
                  "usrSaveDir" => "/tmp/.dbengine",   # Setting of default-value
                  "MainTitle"  => ""                  # Prefix for the main-title
);

#
my $qu;         # Value of CGI-handle

my %parms;      # Hash-Array for input Parameters

my %buttons;            # Feld fuer die Bezeinung und Namen von Buttons. Die
                        # Bezeichnungen (englisch als Key) werden der Sprache
                        # entsprechend eingetragen. analog zum hasharray texte

my %values;     # Hash-Arrays for Data of one Record of one Table
my %fielddesc;  # Description of the attributes and display organization
my %relations;  # Relation between the actual table and the other tables of the database

#
# Definition des Ereignis-Automaten ber zwei hash-Tabellen:
# 1. aufzurufende Quelle (mit required)
# 2. Aufzurufende Funktion
#
# 1. Ereignis = Key, Wert= Aufzurufendes Quellpaket in der Form:
#                          paket
#    oder
#                          paket[(par3,par4,...)]
#    Standardmaessig werden als par1 und par2 Referenzen auf %configs und %parms mitgegeben
#
my %automat_req = (
        'userget'       => 'dbengine_userdefine.pl',
        'userdefine'    => 'dbengine_userdefine.pl',
        'empty'         => '',
        'delete'        => '',
        'confdel'       => '',
        'search'        => '',
        'update'        => '',
        'mupdate'       => '',
        'add'           => '',
        'plain'         => '',
        'print'         => '',
        'back'          => '',
        'menu'          => 'dbengine_crMenu.pl',
        'action'        => 'dbengine_action.pl',
        'ende'          => 0
);

#
# 2. Ereignis = Key, Wert= Aufzurufende Routine
#
my %automat_def = (
        'userget'    => "userget",
        'userdefine' => "userdefine",
        'empty'      => "createStartPage",
        'delete'     => "doDeleteRecord",
        'confdel'    => "doDeleteRecord",
        'search'     => "listTable",
        'update'     => "doUpdateRecord",
        'mupdate'    => "doUpdateMultipleRecords",
        'add'        => "doAddRecord",
        'plain'      => "plainDialog",
        'print'      => "plainDialog",
        'back'       => "plainDialog",
        'menu'       => "crMenu_createMenu",
        'action'     => "action_doAction",
        'ende'       => 0
);


#
# Start der Hauptroutine fr DBENGINE
#
sub start_dbengine {
        # Conf-parameter aus Start in %configs abspeichern
        my $paramv;
        $configs{'refdb'}       = shift;
        $paramv                 = shift;
        $configs{'server'}      = $paramv if $paramv;
        $paramv                 = shift;
        $configs{'port'}        = $paramv if $paramv;
        $paramv                 = shift;
        $configs{'MainTitle'}   = $paramv if $paramv;
        unshift (@INC, $configs{'refdb'}) if $configs{'refdb'};

        require "dbengine_debug.pl";
        require "dbengine_dbase.pl";
        require "dbengine_out_neu.pl";

        &db_db_init(\%parms,\%configs,\%fielddesc,\%relations,\%values); # Initialisierung des Datenbankmoduls
        $qu = new CGI;                                   # Query Initialisieren
        $configs{'qu_cgi'} = $qu;
        &pr_db_init($qu,\%parms,\%configs,\%buttons, \%fielddesc,\%relations,\%values); # Initialisierung des Printmoduls

        $parms{'dbdesc'}    = $configs{'descdb'};
        $parms{'dbdescn'}   = $configs{'descdbn'};
        $parms{'dbsystem'}  = $configs{'dbsystem'};
        #
        # Setting the number of recordsw maximal given in a tablelist
        $configs{'maxrecno'} = 20 unless $configs{'maxrecno'};

        $debug               = $configs{'debug'} if defined $configs{'debug'};
        $configs{'dbsystem'} = "" if ($debug >= 9);
        &dbstart($debug, $debug_co, $debug_file);       # Starten der Moeglichkeit von Debug-Ausschrieben

        &db_db_conndef;                                  # connect to the defining database

        &setConfigs;                                     # Conf-Datei lesen und Daten in %configs abspeichern
        &set_parms;                                      # Parameter aus der Query bernehmen
        &dbstart($debug, $debug_co);                    # Neudefinition der Moegchlichkeit von Debug-Ausschrieben

        # retranslate the value of mode if parms{mode} given by a button
        &pr_db_start;                                    # HTTP- und HTML-Header erzeugen
        my $mode = $parms{'mode'}  = &gbutton($parms{'mode'}) if($parms{'mode'});

        &dbprint_inits(\%configs,\%parms,\%ENV);         # Testauschriebe fr Hash-Speicherungen
        &dbprint_hash(7,"start_dbengine: mode=|$mode|, automat=|$automat_def{$mode}|, parms",\%parms);
        &automat_func($mode);
        exit(0);
}

#
# Subroutine to start a funtion from the automat
#
sub automat_func {
        my $mode     = shift;
        my $rparms   = \%parms;
        my $rconfigs = \%configs;
        my $rqu      = $qu;
        if ($mode && $automat_def{$mode}) {
                if (my $package = $automat_req{$mode}) {
                        my $req_parms = '$rconfigs,$rparms,$rqu';
                        if ($package =~ /^\s*(.+)\((.+)\)\s*$/) {
                                $package   = $1;
                                $req_parms = $2;
                        }
                        my $init_func = $package . '_init';
                        $init_func = $1 . '_init' if ($package =~ /^dbengine_(.+).pl$/);
                        &dbprint(7,"start_dbengine: package=|$package|, init_func=|$init_func|,qu=|$qu|");
                        require $package;
                        dbprint(7,"automat_func: initfunc=|$init_func($req_parms)|");
                        eval "$init_func($req_parms)";
                        &dbError(gtext('sysfehl',gtext('init_func'), "init = '$init_func'", $@)) if $@;
                }
                eval '&' . $automat_def{$mode};
        } else {
                &dbError(gtext('sysfehl',gtext('function'),"mode = '$mode'",gtext('undef')));
        }
}

#
# delete the record $oid
#
sub doDeleteRecord {
        #
        # if the deletion of our record has been confirmed, we'll go ahead and
        # remove it and all of its related records from our database
        if($parms{'mode'} eq "confdel" ) {

                #
                # First lets open the databases: the defining and the ordered
                #
                &db_db_conn(0);

                #
                # first fetch all its relation records that refer to tables
                # which might contain related records that have to be removed
                my $table  = $parms{'table'};
                my $tables = qq{"$table"};
                $tables    = qq{"$1"."$2"} if ($table =~ /^([^\.]+)\.([^\.]+)$/);
                my $oid    = $parms{'oid'};
                my %childs;

                #
                # get the field types
                # an get further Information for checking the possibility to update this row
                &db_getFieldTypes;
                &db_get_row($table,$oid);
                &db_get_childs($table,\%childs);

                my $value;
                my $errorg = 0;

                foreach my $name (keys %values) {
                        if ( $name ne 'tmin' ) {
                                $errorg += &db_check_child($name, $values{$name} . "xxx" , \%childs);
                        }
                }
                &dbError(&gtext('delrerr')) if $errorg; # Fehler in check_child gesammelt, damit nicht
                                                        # nur ein Fehler gemeldet wird. Hier dann Abbruch
                #
                # then remove the record
                my $cmd = qq{DELETE FROM $tables WHERE oid=$oid};
                &dbprint(1, "<table><tr><td>" . gtext('cmd') . ":</td><td>$cmd</td></tr><tr><td>" .
                            gtext('binds') . ":</td><td>&nbsp;</td></tr></table>");

                if (&db_check_row($parms{'oid'})) {
                        &db_do_cmd('dbase',$cmd,undef,1);
                        &pr_listTable;
                }

                &db_db_disc;    # Ending the database-connections

        #
        # if our user just asked for the deletion we'll ask him to confirm his request
        } else {
                &pr_form_start(&gtext('delconfirm'));
                #
                # First get all parameters for the confirmation asking
                foreach my $name ($qu->param) {
                        print $qu->hidden($name,$parms{$name}) . "\n" unless ( $name eq 'mode' );
                }

                #
                # finally complete our form
                print "<TABLE><TR><TD>" . &gtext('reallydel') . "</TD><TD>";

                print $qu->submit(-name=>'mode', -value=>&sbutton('confdel')) . "</TD></TR>\n";

                print "<TR><TD>" . &gtext('nodelrec') . "</TD><TD>";
                print $qu->submit(-name=>'mode', -value=>&sbutton('back')) . "</TD></TR></TABLE>\n";
                print $qu->endform . "\n";
                &pr_db_finish;
        }
}

#
# add the complete record that lives in $query to $table
# therefore first of all get the fieldtypes for $table
#
sub doAddRecord {
        #
        # First lets open the databases: the defining and the ordered
        #
        &db_db_conn(0);

        my $table = $parms{'table'};
        my $tables = qq{"$table"};
        $tables    = qq{"$1"."$2"} if ($table =~ /^([^\.]+)\.([^\.]+)$/);
        my $setclause = "(";
        my $valueclause = "VALUES (";
        my @bind_values;
        my $errorg = 0;
        my $nullfields = 0;

        #
        # then get the field types and start to build our SQL statement
        &db_getFieldTypes;

        foreach my $nameq ($qu->param) {
                if ($nameq =~ /^__(.+)$/) {
                        my $name = $1;
                        my $fielddisplay = $fielddesc{$name}{'DISPLAY'};
                        unless ( ($name eq "oid" || $name eq "tmin") &&
                                 ($fielddisplay && $fielddisplay eq 'SERIAL')) {

                                my ($errorf,$content) = &pr_checkFieldContent($name, $qu->param($nameq));
                                $errorg              += $errorf;
#                                &dbprint(7,"doAddRecord: name=|$name|, content=|$content|, default=|$fielddesc{$name}{'DEFAULT'}|");
                                if ( length($content) == 0 ) {
                                        $nullfields += 1;
                                        unless ($fielddesc{$name}{'DEFAULT'}) {
                                                $setclause   .= qq{ \"$name\",};
                                                $valueclause .= qq{ NULL ,};
                                        }
                                }  else {
                                        $setclause   .= qq{ \"$name\",};
                                        $valueclause .= qq{ ? ,};
                                        push(@bind_values, &pr_local2SQLformat($content, $fielddesc{$name}{'TYPE'}));
                                }
                        }
                }
        }
        &dbError(gtext('inperr')) if $errorg; # Fehler in checkFieldContent gesammelt, damit nicht
                                              # nur ein Fehler gemeldet wird. Hier dann Abbruch
        if ( $fielddesc{'tmin'}{'TYPE'} ) {
                $setclause .= " tmin)";
                $valueclause .= "'now')";
        } else {
                # remove the trailing ", " and set the trailing ")"
                chop($setclause); $setclause .= ")";
                chop($valueclause); $valueclause .= ")";
        }
        #
        # and then build our insert command
        my($cmd) = qq{INSERT INTO $tables $setclause $valueclause};

        dbprint_cmd_binds(1,$cmd, @bind_values);
        &dbprint (1,&gtext('nfields',$nullfields)) if ( $nullfields );

        #
        # and send it to the database
        &db_do_cmd('dbase',$cmd,\@bind_values,1);
        &pr_listTable;

        &db_db_disc;    # Ending the database-connections
}

#
# save the given values from $query in the record $oid
#
sub doUpdateRecord {
        #
        # First lets open the databases: the defining and the ordered
        #

        &dbprint_hash(7,"doUpdateRecord: parms",\%parms);
        &db_db_conn(0);

        my $table = $parms{'table'};
        my $tables = qq{"$table"};
        $tables    = qq{"$1"."$2"} if ($table =~ /^([^\.]+)\.([^\.]+)$/);
        my $oid   = $parms{'oid'};
        my %childs;

        #
        # get the field types
        # an get further Information for checking the possibility to update this row
        &db_getFieldTypes;
        &db_get_row($table,$oid);
        &db_get_childs($table,\%childs);

        # Build the UPDATE command and check the input fields on correct input
        my $setclause = "SET ";
        my @bind_values;
        my $errorg = 0;
        my $nullfields = 0;

        #
        # add assignments for all given values
        foreach my $nameq ($qu->param) {
                my $errorf;
                if ( $nameq =~ /^__(.+)$/ ) {
                        my $name = $1;
                        next if($fielddesc{$name}{'DISPLAY'} && $fielddesc{$name}{'DISPLAY'} eq 'SERIAL');
                        my $content;
                        ($errorf, $content) = &pr_checkFieldContent($name, $qu->param($nameq));
                        unless ( $errorf ) {
                                &dbprint(7,"doUpdateRecord: length(content)=|%d|, length(parm)=|%d|",
                                           length($content), length($parms{$nameq}));
                                if ( length($content) == 0 ) {
                                        $nullfields = 1;
                                        $setclause .= qq{"$name" = NULL ,};
                                }  else {
                                        $setclause .= qq{"$name" = ? ,};
                                        push(@bind_values, &pr_local2SQLformat($content, $fielddesc{$name}{'TYPE'}));
                                }
                        } else {
                                $errorg = 1;
                        }
                        $errorg += &db_check_child($name, $content, \%childs);
                }
        }
        &dbError(gtext('inperr')) if $errorg; # Fehler in checkFieldContent gesammelt, damit nicht
                                              # nur ein Fehler gemeldet wird. Hier dann Abbruch
        #
        # now check the values if they are used in child-tables
        #
        # and send it to the database
        my $cmd;

        if ( $fielddesc{'tmin'}{'TYPE'} ) {
                $cmd = qq{UPDATE $tables $setclause "tmin"='now' WHERE "oid" = $parms{'oid'}};
        } else {
                $setclause = $1 if ( $setclause =~ /^(.+)\s*,$/ );
                $cmd = qq{UPDATE $tables $setclause WHERE "oid" = $parms{'oid'}};
        }
        &dbprint(7,"doUpdateRecord: cmd=|$cmd|,\nsetclause=|$setclause|,\nbindvalues=|@bind_values|");
        dbprint_cmd_binds(1,$cmd, @bind_values);
        &dbprint (1,&gtext('nfields',$nullfields)) if ( $nullfields );

        if (&db_check_row($parms{'oid'})) {
                &db_do_cmd('dbase',$cmd,\@bind_values,1);
                &pr_listTable;
        }

        &db_db_disc;    # Ending the database-connections
}

#
# From listTable it is possible to do a multiple update ( for all rows in the table)
sub doUpdateMultipleRecords {
        # First lets open the databases: the defining and the ordered
        #
        &db_db_conn(0);

        my $table = $parms{'table'};
        my $tables = qq{"$table"};
        $tables    = qq{"$1"."$2"} if ($table =~ /^([^\.]+)\.([^\.]+)$/);

        my $oid   = $parms{'oid'};
        my %childs;
        my $field = $parms{'fieldname'};
        my $value = $parms{'fieldvalue'};

        #
        # get the field types
        # an get further Information for checking the possibility to update this row
        &db_getFieldTypes;
        &db_get_childs($table,\%childs);

        # Build the UPDATE command and check the input fields on correct input
        my $errorf = 0;
        my $errorg = 0;
        ( $errorf, $value ) = &pr_checkFieldContent($field, $value);
        &dbError(gtext('mupdateerror')) if $errorf;

        #
        # add assignments for all given values
        foreach my $nameq ($qu->param) {
                if ($nameq =~ /^OID(\d+)$/ ) {
                        my @bind_values;
                        my $oid = $1;
                        my $wert = &pr_local2SQLformat($value, $fielddesc{$field}{'TYPE'});
                        unless (&db_check_child($field, $wert, \%childs)) {
                                my $hv = "?";
                                unless ($wert) {
                                        $hv = "NULL";
                                } else {
                                        push(@bind_values, $wert);
                                }
                                my $cmd = "UPDATE $tables SET \"$field\" = $hv WHERE oid=$oid";
                                &dbprint(7,"doUpdateMultipleRecords: oid=|$oid|, nameq=|$nameq|, cmd=|$cmd|," .
                                           " bind_values=|@bind_values|");
                                if (&db_check_row($oid)) {
                                        &db_do_cmd('dbase',$cmd,\@bind_values,0);
                                }
                        } else {
                                $errorg++;
                        }
                }
        }
        &dbError(gtext('mupdateerror')) if $errorg; # Fehler in checkFieldContent gesammelt, damit nicht
        &db_do_commit;
        &pr_listTable;

        &db_db_disc;    # Ending the database-connections
}

#
# let's generate a list with all records in $table
# beginning with the headline of our window
sub listTable {
        #
        # First lets open the databases: the defining and the ordered
        #
        &db_db_conn(1);

        #
        # List the relation with the given data
        $parms{'search'} = 0 unless $parms{'oldsearch'}; # No old search will be used
        &db_getFieldTypes;
        &pr_listTable;

        &db_db_disc;    # Ending the database-connections
}


#
# let's generate a dialog that allows for entering values to
# either add new records or search for existing ones
#
sub plainDialog {

        #
        # First lets open the databases: the defining and the ordered
        #
        &db_db_conn(1);

        $parms{'printm'} = 1 if ( $parms{'mode'} eq 'print');

        &db_getFieldTypes;      # Filling the hasharrays from datas of database
        &db_fillValues;

        &dbprint_hash_hash(8,"plainDialog: fielddesc",\%fielddesc);
        &dbprint_hash(8,"plainDialog: values",\%values);
        &dbprint(7,"plainDialog: Aufruf des Formulars");

        &pr_create_dialog;      # creating the table mask
        &pr_db_stop_o_mode;     # Ending the Form and page
        &db_db_disc;            # Ending the database-connections

        return;
}

#
# Creating the startpage of the body-frame (default is empty)
#
sub createStartPage {
        my $mode = $configs{'start_mode'};
#        &dbprint_hash(7,"createStartPage: configs", \%configs);
#        &dbprint_hash(7,"createStartPage: automat_def=|\%automat_def|, automat_def",\%automat_def);
        if ($mode && $mode ne 'empty') {
                $parms{'mode'} = $mode;
                $parms{$configs{'start_fname'}} = $configs{'start_fvalue'}; # fname=fieldname, fvalue=fieldvalue
                &dbprint_hash(7,"createStartPage: automat_def",\%automat_def);
                &automat_func($mode);
        } else {
                if (my $db_desc = $configs{'comment'}) {
                        &pr_list_start(&gtext('dbasename') . ": $parms{'dbasen'}");
                        print "<P><BR><H2><CENTER>$db_desc</CENTER></H2><BR><P>\n";
                }
                &pr_db_finish;
        }
}

#
# Setting the rest of the configs, starting debugging Posibilities
#
sub setConfigs {
        #
        # Read the Config-File and store the values in genConfig
        #

        my $confcmd = "SELECT index,values FROM configs";
        my $stdh    = &db_db_get('dbdesc',$confcmd);
        while (my $rarray = &db_db_hashref($stdh) ) {
                my $index = $$rarray{'index'};
                my $configDate = $$rarray{'values'};
                &dbprint_hash(10,"setConfigs: rarray",$rarray);
                $configs{$index} = $configDate;
        }
        $stdh->finish;

        #
        # Getting the configs from the Environment
        #
        my @lang = split(/,/,$ENV{'HTTP_ACCEPT_LANGUAGE'});
        foreach my $lang (@lang) {
                $lang = $1 if ($lang =~ /^\s*([^\s;]+);/ );
                &dbprint(11,"setConfigs: env=|$ENV{'HTTP_ACCEPT_LANGUAGE'}|, lang=|$lang|");
                if($configs{'lang'}) {
                        $configs{'langs'} .= ",$lang";
                } else {
                        $configs{'lang'} = $configs{'langs'} = $lang;
                }
        }
        &dbprint_hash(10,"setConfigs: configs",\%configs);
}

#
# Setting and saving some parameters coming in the call or saved in an user-specific file
# Setting also the configs defined database specific
#
sub set_parms {
        #
        # Getting the params of the query
        #
        my @parkeys = $qu->param;
        foreach my $parkey (@parkeys) {
                $parms{$parkey} = $qu->param($parkey);
        }

        $parms{'dbase'}         = $configs{'defdb'} if (!$parms{'dbase'} && $configs{'defdb'});
        $parms{'dbasen'}        = $configs{'defdbn'} if (!$parms{'dbasen'} && $configs{'defdbn'});
        $parms{'mode'}          = 'userget' unless $parms{'mode'};

        $values{'tmin'}         = $qu->param('tmin');        # (optional) tmin of record to maintain

        #
        # Getting the saved userspezific datas if possible
        #
        &get_user_file if ($parms{'kennz'});
        # correcting the dbdesc for using with arbitrary Drivers
        # database 'dbinfo' must use the same driver as the 'dbase' database
        if ($parms{'dbase'} && !$parms{'dbsystem'}) {
                $parms{'dbsystem'} = &dbase_get_drivername(\$parms{'dbase'});
        }
        if($parms{'dbsystem'}) {
                $parms{'dbase'} = &dbase_get_dbname($parms{'dbasen'}) if ($parms{'dbasen'} && !$parms{'dbase'});
                $parms{'dbdesc'} = &dbase_get_dbname($parms{'dbdescn'}) if ($parms{'dbdescn'} && !$parms{'dbdesc'});
        }

        $parms{'recno'} = 0 unless $parms{'recno'};

        if(my $dbase = $parms{'dbase'}) { # Getting the Database specific configs
                my $confcmd = "SELECT * FROM databaseconfigs where dbasename = '" . &dbase_get_name($dbase) . "'";
                my $stdh    = &db_db_get('dbdesc',$confcmd);
                while (my $rarray = &db_db_hashref($stdh) ) {
                        my $configname = $$rarray{'configname'};
                        my $configvalue = $$rarray{'configvalue'};
                        if ( $configname eq 'schemas') {              # special action for user-specific schemata
                                foreach my $schem_user (split(/;/,$configvalue)) {
                                        my ($user,$schemas) = split(/:/,$schem_user);
                                        if ($user eq $parms{'user'}) {
                                                $configs{'schemas'} = $schemas;
                                                last;
                                        }
                                }
                        } else {
                                $configs{$configname} = $configvalue;
                        }
                }
                $stdh->finish;
        }
        $debug = $configs{'debug'} if ($configs{'debug'} && $configs{'debug'} > $debug);

}


#
# Getting some specific Data for use from earlyer calls
#
sub get_user_file {
        my ($alterg,$rest) = split(/_/,$parms{'kennz'});
        my $erg = 0;
        my $searchfound = 0;
        my $bindsfound  = 0;
        &dbprint(7,"get_user_file: userSaveDir=|" . $configs{'usrSaveDir'} . "|");
        &dbError(&gtext('unzben')) unless (-r "$configs{'usrSaveDir'}/$rest");
        &crypt_start;
        if(open(KENNZ,"<$configs{'usrSaveDir'}/$rest")) {
                while (my $satz = <KENNZ>) {
                        chomp($satz);
                        next unless $satz;
                        my @werte = split(/\|\|\|/,&name_decrypt($satz));
                        &dbprint(8,"get_user_file: werte=|@werte|");
                        foreach my $wert1 (@werte) {
                                my ($key,$wert) = split(/=/,$wert1,2);
                                $key = $1 if ( $key =~ /^\s*([^\s]+)\s*$/ );
                                $wert = $1 if ( $wert =~ /^\s*"(.+)"\s*$/ );
                                &dbprint(10,"get_user_file: key=|$key|, wert=|$wert|");
                                if ( $parms{$key} && $wert ne $parms{$key}) {
                                        &dbprint(10,"get_user_file: Fehlwert mit parms_key = |$parms{$key}| und wert=|$wert|");
                                        &dbError(&gtext('unzuser'));
                                }
                                if ($wert && (!$parms{$key} || $key eq 'search' || $key eq 'binds')) {
                                        $searchfound = 1 if($key eq 'search');
                                        $bindsfound  = 1 if($key eq 'binds');
                                        $parms{$key} = $wert;
                                }
                                $erg++;
                                last if ( $erg == $alterg);
                        }
                }
                close(KENNZ);
        } else {
                dbError(&gtext('unzben')) unless (-r "$configs{'usrSaveDir'}/$rest");
        }
        &crypt_stop;
        &dbError(&gtext('unzben')) if ($erg != $alterg  &&  $erg < 6);
        $parms{'binds'} = "" if ($searchfound && !$bindsfound);
}

#
# Saving some specific Data for using in later calls
#
sub set_user_file {
        my %werte;
        my $kennz;

        if( $kennz = $parms{'kennz'}) {
                my $anz;
                ($anz,$kennz) = split(/_/,$kennz);
        }  else {
                $werte{'pid'}   = $$;
                if($parms{'ktime'}) {
                        $werte{'ktime'} = $parms{'ktime'};
                } else {
                        $werte{'ktime'} = sprintf("%d",time);
                        $parms{'ktime'} = $werte{'ktime'};
                }
                $kennz = $werte{'ktime'} . "-" . $werte{'pid'};
        }
        my $anz = 0;
        &dbprint(7,"set_user_file: usrSaveDir=|$configs{'usrSaveDir'}|");
        mkdir($configs{'usrSaveDir'}) unless (-d $configs{'usrSaveDir'});

        &dbError(&gtext('unznot')) unless (open(KENNZ,">$configs{'usrSaveDir'}/$kennz"));

        &crypt_start;
        my $satz;
        foreach my $key ('pwd', 'user', 'dbsystem', 'dbdesc', 'dbdescn', 'dbasen','schemas','search', 'binds') {
                if ($parms{$key}) {
                        $satz .= "|||" if $satz;
                        $satz .= "$key = \"$parms{$key}\"";
                        $anz++;
                }
        }
        &dbprint(9,"set_user_file: satz=|$satz|");
        print KENNZ &name_crypt($satz) . "\n";
        &crypt_stop;

        close(KENNZ);

        return("${anz}_$kennz");
}

1;
