############################
#
# dbengine_dbase.pl
#
# Modul zu dbengine.cgi: Verwaltung und Nutzung von Datenbanken ber
# Browserschnittstelle.
#
# Version 2.3.2
# <c> 2003, 2004 Detlef Duerr
# get closer Information from email Detlef.Duerr@gmx.de
# Version 1.01
# <c> 1997, 1998, 1999 Detlef Duerr
# get closer Information from email Detlef.Duerr@atm-computer.de
# older version
# <c> 1996 Ingo Ciechowski
# see closer information at http://www.cis-computer.com/dbengine
#
#
#  Database Library for dbengine.pl new version
#
#
###########################

use strict;

my $rparms;
my $rconfigs;
my $rfielddesc;
my $rrelations;
my $rvalues;

my $dbase;
my $dbasen;
my $dbsystem;
my $table;

my $dbdefh;
my $dbordh;

my %dbconns;

#
# Initialisierung des Moduls
#
sub db_db_init {
        $rparms         = shift;
        $rconfigs       = shift;
        $rfielddesc     = shift;
        $rrelations     = shift;
        $rvalues        = shift;

        $dbase          = $$rparms{'dbase'} if $$rparms{'dbase'};
        return;
}

#
# Opening the defining (default=dbinfo) database
#
sub db_db_conndef {
#        &dbprint(11,"db_db_conn: dbinfo-conn-statement = |DBI->connect($$rparms{'dbdesc'}), $$rparms{'user'}, $$rparms{'pwd'}," .
#                   "{ PrintError => 1, AutoCommit => 1 } )|");

        # connect to the ordered database
        $dbdefh = DBI->connect($$rparms{'dbdesc'}, $$rparms{'user'}, $$rparms{'pwd'},
                               { PrintError => 1, AutoCommit => 1 } );
        &dBaseError($DBI::errstr,"DBI->connect",&gtext('noconnect',&dbase_get_name($$rparms{'dbdesc'}))) unless $dbdefh;
        $dbconns{'dbdesc'} = $dbdefh;
        $dbsystem          = $$rparms{'dbsystem'} if $$rparms{'dbsystem'};

}

#
# Opening the ordered database
#
sub db_db_conn {
        my $conn_art = shift;
        $dbordh = &db_db_aconn('dbase',$conn_art,$$rparms{'dbase'});
        &dbprint(9,"db_db_conn: dbdefh=|$dbdefh|, dbordh=|$dbordh|");

        if (my $schemas = $$rparms{'schemas'}) {
                my $rc = $dbordh->do("SET search_path TO $schemas");
                &dBaseError($dbordh->errstr,"SET search_path TO $schemas", &gtext('dberror')) unless $rc;
        }

        # after getting the connection save some hashvalues in local values
        $dbase  = $$rparms{'dbase'} if $$rparms{'dbase'};
        $dbasen = &dbase_get_name($dbase);
        $table  = $$rparms{'table'} if $$rparms{'table'};

        return;
}

sub db_set_search_path {

}

#
# Connecting to a database
#
sub db_db_aconn {
        my ($dbx,$conn_art,$dbnsname) = @_;
        $$rparms{$dbx} = &dbase_get_dbname($dbnsname) if ( $dbx ne 'dbase');

#        my $tracakt = DBI->trace('/tmp/..db_debug.txt');
#        $DBI::TraceLevel = "4|All";
        my $db_dr_name = $$rparms{$dbx};   # driver-specific name of the ordered database
        &dbprint(9,"db_db_aconn: dbase-conn-statement = |DBI->connect($db_dr_name, $$rparms{'user'}, $$rparms{'pwd'}," .
                        "{ PrintError => 1, AutoCommit => $conn_art, ChopBlanks => 1, FetchHashKeyName => \"NAME\" } )|");

        # connect the ordered databes with AutoCommit = conn_art
        my $dbordn = DBI->connect($db_dr_name, $$rparms{'user'}, $$rparms{'pwd'},
                               { PrintError => 1, AutoCommit => $conn_art, ChopBlanks => 1, FetchHashKeyName => "NAME"} );
        unless ( $dbordn ) {
          my $errstr = $DBI::errstr;
          &dbprint(9, "db_db_aconn: Errorstring=|$errstr|");
          &dBaseError($errstr,"DBI->connect",&gtext('noconnect') . ". DBname: " . &dbase_get_name($db_dr_name));
        }

        $dbconns{$dbx} = $dbordn;

        return ($dbordn);
}

#
# Getting the typeinfo for a specific datatype
#
sub db_get_typeinfo {
        my $data_type = shift;
        my $erg = $dbordh->type_info($data_type);
        $erg = $$erg{'TYPE_NAME'};
        unless ($erg) {
                if ($data_type == 26) {
                        $erg = 'oid';
                } elsif ($data_type == 1700) {
                        $erg = 'numeric';
                }
        }
}

#
# Ending the database connections
#
sub db_db_disc {
        $dbdefh->disconnect if $dbdefh;
        $dbordh->disconnect if $dbordh;
}

#
# Gettin a array of all schemas of the ordered database
#
sub db_db_schemas {
        my $user     = shift;
        my $pathconf = shift;
        # aktuell nicht ber tableinfo holen, sondern mit Zugriff auf Systemtabellen
        # nur so ist z.Z. der 'owner' eines Schemas feststellbar
        # "my $sth = $dbordh->table_info('','%','','');" liefert nur schemanamen.
        # my $schemas = $sth->fetchall_arrayref or dBaseError($DBI::errstr,"dbordh->table_info",&gtext('noschemas'));
        my $cmd = qq{SELECT nspname,usename,nspacl FROM pg_namespace, pg_user WHERE nspname !~* '^pg_' AND usesysid = nspowner};
        &dbprint(7,"db_db_schemas: user=|$user|, pathconf=|$pathconf|");
        my $sth = &db_db_get('dbase',$cmd);
        my @schemas;
        my $userp = &db_is_admin($user);
        my $userconf = "";
        $userconf    = $2 if ($pathconf =~ /^(.+;)*\s*$user\s*:\s*([^;]+)\s*(;.+)*$/);
        my $userall  = (!$userconf ||($pathconf =~ /^(.+;)*\s*($user|All)\s*:\s*([^;]+,\s*)*ALL\s*([,;].+)*$/));
        if ($pathconf =~ /^(.+;)*\s*ALL\s*:\s*([^;]+)\s*(;.+)*$/) {
                $userconf .= "," if $userconf;
                $userconf .= $2;
        }
        my $admconf  = "";
        my $admall   = 0;
        if ($userp) {
                $admconf = $2 if ($pathconf =~ /^(.+;)*\s*Admins\s*:\s*([^;]+)\s*(;.+)*$/);
                $admall  = (!$admconf || ($pathconf =~ /^(.+;)*\s*Admins\s*:\s*([^;]+,\s*)*ALL\s*([,;].+)*$/));
        }
        &dbprint(7,"db_db_schemas: sth=|$sth|, userp=|$userp|, userall=|$userall|, admall=|$admall|");
        while (my $rarray = &db_db_hashref($sth)) {
                my $sch_name = $$rarray{'nspname'};
                my $sch_own  = $$rarray{'usename'};
                my $sch_acl  = $$rarray{'nspacl'};
                &dbprint(7,"db_db_schemas: sch_name=|$sch_name|, admconf=|$admconf|, userconf =|$userconf|");
                &dbprint_hash(7,"db_db_schemas: rarray",$rarray);
                if ( $sch_own eq $user ||
                     ($admall || $admconf =~ /^([^,]+,\s*)*$sch_name\s*(,.+)*$/) ||
                     (($sch_acl && $sch_acl =~ /(({)|(,$user))=((UC)|U|C)(,|})/ ) &&
                           ($userall || $userconf =~ /^([^,]+,\s*)*$sch_name\s*(,.+)*$/))
                   ) {
                        push(@schemas,$sch_name);
                }
                &dbprint_array(7,"db_db_schemas: schemas",\@schemas);
        }

        return (@schemas);
}

#
# Get the owner of a database or e schema
#
sub db_db_get_owner {
        # No inputparameter; output = (db_owner, sch_owner)
        my $database = $$rparms{'dbasen'};
        my $schema   = $$rparms{'schemas'};
        my @erg = ();
#        &dbprint(7,"db_db_get_owner: database=|$database|, schema=|$schema|");

        if ($database) {
                my $cmd = "select usename as owner from pg_database a, pg_user b " .
                          "where a.datname = '$database' AND a.datdba = b.usesysid";
                my $sth = &db_db_get('dbase',$cmd);
                push(@erg, &db_db_hashref($sth)->{'owner'});
                $sth->finish;
        }
        if ($schema && !( $schema =~ /,/)) {
                my $cmd = "select usename as owner from pg_namespace a, pg_user b " .
                          "where nspname = '$schema' AND a.nspowner = b. usesysid";
                my $sth = &db_db_get('dbase',$cmd);
                push(@erg, &db_db_hashref($sth)->{'owner'});
                $sth->finish;
        } else {
                push(@erg,''); # there is no single schema and therefore no schema-owner
        }
        &dbprint_array(7,"db_db_get_owner: database=|$database|, erg",\@erg);
        return(@erg);
}

#
# Gettin a array of all tablenames of the ordered database
#
sub db_db_tables {
        # no input parms, returns array of tablenames
        my $user    = $$rparms{'user'};
        my $schema  = $$rparms{'schemas'};
        $schema = "" unless $schema;
        my @tnames = ();

        unless ($schema) {
#                my $cmd = "SHOW search_path";
#                my $sterg = &db_db_get('dbase',$cmd);
#                my $rerg = $sterg->fetchall_arrayref;
#                my @schemas  = split(/,/,$$rerg[0][0]);
#                $sterg->finish;
                my @schemas =split(/,/,&db_show_variable('search_path'));
                $schema = "";
                foreach my $sch_n (@schemas) {
                        $sch_n = $1 if ( $sch_n =~ /^\s*(\S+)\s*$/ );
                        $sch_n = $user if ( $sch_n =~ /^\$user$/ );
                        $schema .= "," if $schema;
                        $schema .= $sch_n;
                }
        }
        my $sth = $dbordh->table_info("","",'',"'TABLE','VIEW'");
        dbprint(7,"db_db_tables: schema=|$schema|, sth=|$sth|");
        my $rrarray = $sth->fetchall_arrayref or dBaseError($DBI::errstr,"dbordh->table_info",&gtext('notables'));
        foreach my $rarray (@$rrarray) {
                my $schema_t = $$rarray[1];
                my $table_t  = $$rarray[2];
                next if ($table_t =~ /^pg_/ );
                next unless ($schema =~ /^(.+,)*$schema_t(,.+)*$/);
                dbprint_array(7,"db_db_tables: schema=|$schema|, rarray",$rarray);
                $table_t = "$schema_t.$table_t" unless ($schema_t eq "public");
                push(@tnames, $table_t);
        }
        $sth->finish;
        &dbprint_hash(7,"db_db_tables: tnames=|" . @tnames . "|, names=|@tnames|, parms",$rparms);
        return (@tnames);
}

#
# Subroutine getting a variable from dtabasesystem
#
sub db_show_variable {
        my $variable = shift;
        my $cmd = "SHOW $variable";
        my $sterg = &db_db_get('dbase',$cmd);
        my $rerg = $sterg->fetchall_arrayref;
        my $ergebnis = $$rerg[0][0];
        $sterg->finish;
        return($ergebnis);
}

#
# this routine does the cmd (update, delete, insert) given by the parameter
#
sub db_do_cmd {
        my ($dbx, $cmd, $rbind_values, $commit) = @_;
        my $dbh = $dbconns{$dbx};
        &dbprint(7,($rbind_values ? "db_do_cmd: dbase=|$dbx|, handle=|$dbh|, cmd=|$cmd|,\nbinds=|@$rbind_values|"
                                  : "db_do_cmd: dbase=|$dbx|, handle=|$dbh|, cmd=|$cmd|" ));
        my $rc = ( $rbind_values ? $dbh->do($cmd, undef, @$rbind_values) : $dbh->do($cmd) );
        &dBaseError($dbh->errstr,$cmd,&gtext('doerror',&dbase_get_name($$rparms{'dbase'}))) unless $rc;
        &db_do_commit if ($commit);
        return $rc;
}

#
# Routine zur Durchfhrung der eigentlichen Update, Delete, Insert-Aufgabe
#
sub db_do_commit {
        my $dbh = $dbconns{'dbase'};
        my $rc  = $dbh->commit;
        &dBaseError($dbh->errstr," ",&gtext('commiterror', &dbase_get_name($$rparms{'dbase'}))) unless $rc;
        $dbh->{AutoCommit} = 1;
        &dbprint(9,"db_do_cmd: AutoCommit=|" . $dbh->{AutoCommit} . "|");
}

#
# Searching Data from database
#
sub db_db_get {
        my ($database, $cmd, $rbind_values)  = @_;

        &dbprint(11,"db_db_get: $database-prepare-statement=|$dbconns{$database}->prepare($cmd)|");
        &dbprint(11,"db_db_get: $database-binds=|@$rbind_values|") if ($rbind_values);

        my $ergp = $dbconns{$database}->prepare($cmd)
                   || &dBaseError($dbconns{$database}->errstr,$cmd,&gtext('preperror',&dbase_get_name($$rparms{$database})));
        my $erge;
        if( $rbind_values && $$rbind_values[0] ) {
                &dbprint(11,"db_db_get: $database-exec-statement=|\$ergp->execute|, bindsanz=|@$rbind_values| mit binds=|" .
                            @$rbind_values . "|");
                $erge = $ergp->execute(@$rbind_values);
        } else {
                &dbprint(11,"db_db_get: $database-exec-statement=|\$ergp->execute| mit ergp=|$ergp|");
                $erge = $ergp->execute;
        }
        &dBaseError($ergp->errstr,$cmd, &gtext('execerror',&dbase_get_name($$rparms{$database}))) unless $erge;
        &dbprint(11,"db_db_get: Ende mit erge=|$erge|");
        return($ergp);
}

#
# Getting a hashref for Data from a Database
#
sub db_db_hashref {
        my $stdh = shift;
        my $referg = $stdh->fetchrow_hashref;
        &dBaseError($stdh->errstr,"\$sth->fetchrow_hashref mit Meldung: |" . $stdh->err . "|", &gtext('getfetchr')) if $stdh->err;
        return($referg);
}

#
# Getting the row of the table $table for the actual oid
#
sub db_get_row {
        my ($table, $oid) = @_;

        my $cmd = qq{SELECT oid, a.* FROM "$table" a WHERE "oid"=$oid};

        my $result = &db_db_get('dbase',$cmd);
        my $rarray = &db_db_hashref($result);
        &dbprint_hash(9,"db_get_row: cmd=|$cmd|, rarray",$rarray);

        foreach my $name (keys %$rarray) {
                my $wert = $$rarray{$name};
                $wert = &xChop($wert) if $wert;
                $$rvalues{$name} = &db_SQL2value($name, $wert);
        }
        &dbprint_hash(9,"db_get_row: values", $rvalues);
        $result->finish;
}

#
# Getting the number of rows for a specific statement
#
sub db_db_get_nrows {
        my ($database, $cmd, $rbind_values) = @_;

        my $stdh = &db_db_get($database,$cmd, $rbind_values);
        my $nrows = $stdh->rows;

        #
        # return the no. of rows and the statementhandle for fetching the rows of data
        return ($nrows,$stdh);
}

#
# Getting a array from a list of values for a given key in $rconfigs
#
sub get_db_array {
        my $key = shift;
        return(undef) if( !$$rconfigs{$key} || $$rconfigs{$key} eq '()');

        my @dbs = split(/,/,$$rconfigs{$key});
        my @dbs1;

        foreach my $dbs (@dbs) {
                $dbs = $1 if ($dbs =~ /^\s*"(.+)"[^"]*$/ );
                push (@dbs1,$dbs);
        }

        dbprint(9,"get_db_array: key=|$key|, array=|@dbs|");
        return(@dbs1);
}

#
# Check the usernames if allowed or not (means: containing actual user)
#
sub db_check_usernames {
        my ($username,$db_owner,$sch_owner)  = @_;
#        dbprint(7,"db_check_usernames: db_owner=|$db_owner|, sch_owner=|$sch_owner|, usernames=|$username|");

        return (1) unless $username;  # no given usernames means allowed for everybody

        my @usernames = split(/,/,$username);
        @usernames = ('ALL') unless ( @usernames ); # if there no usernames given for all users it is allowed
        my $user = $$rparms{'user'};
        dbprint_array(7,"db_check_usernames: user=|$user|, usernames",\@usernames);
        foreach $username (@usernames) {
                $username = $1 if ($username =~ /^\s*([^\s].*)\s*$/);
                $username = $1 if ($username =~ /^([^#]*)#/); # there is a comment after the username
                next unless $username; # in the line of users is a blank name, means nothing

                return(1) if ($username eq 'ALL');  # ALL users are allowed
                return(1) if ( $username eq 'DB_OWNER'  && $user eq $db_owner ); # user is allowed if owner of the database
                return(1) if ( $username eq 'SCHEM_OWNER' && $user eq $sch_owner ); # user is allowed if owner of actual schema

                if ( $username =~ /^admins$/i ) {   # All admins are allowed user
                        return(1) if (&db_is_admin($user));
                }

                return(1) if ($user eq $username );  # the actual user is allowed
        }

        return(0);   # actual user not allowed
}

#
# Subroutine for checking user is admin
#
sub db_is_admin {
        my $usern = shift;
        return ($$rconfigs{'admins'} =~ /(.+,)*$usern(,.+)*/);
}

#
# Get the displayname of a table or action
#
sub db_get_displayname {
        my ($fieldvalue,$fieldname,$tablename,$displayfield) = @_;
        my $dispn = "";
        my $cmd = "SELECT $displayfield as display FROM $tablename WHERE dbasename='$dbasen' AND $fieldname = '$fieldvalue'";
        &dbprint(7,"db_get_displayname: cmd=|$cmd|");
        my $erg = &db_db_get('dbdesc',$cmd);
        if (my $rarray = &db_db_hashref($erg)) {
                $dispn  = $$rarray{'display'};
                $erg->finish;
        }
#        &dbprint(7,"db_get_displayname: fieldvalue=|$fieldvalue|, dispn=|$dispn|");
        return(&db_get_disptext($dispn,$fieldvalue));
}

#
# Getting the displaytext of a table
#
sub db_get_tabledisplay {
        my $table = shift;
        return(&db_get_displayname($table,'tablename','basedesign','displayname'))
}

#
# Getting the displaytext
#
sub db_get_disptext {
        my ($dispn,$fieldvalue) = @_;
        $dispn = &upfirst($fieldvalue) unless $dispn;
        $dispn = &gtext($1) if ($dispn =~ /^lang\((.+)\)$/ );
        return ($dispn);
}

#
# Datenbanknamen aus dbi-Datasource ermitteln
#
sub dbase_get_name {
        my $elem = shift @_;
        my $name;

        if ($elem =~ /^[^:]+:[^:]+:([^;\@]+)([\@;]|$)/ ) {
                $elem = $1;
        }
        if ($elem =~ /^[^=]*=(.+)$/) {
                $name = $1;
        } else {
                $name = $elem;
        }
        return ($name);

}

#
# Driverparts aus dbi-Datasource ermitteln
#
sub dbase_get_drivername {
        my $elem        = shift;
        my $driver;

        $driver = $1 if ($elem =~ /^\s*dbi:([^:]+):/ );
        &dbprint(10,"dbase_get_drivername: elem=|$elem|, driver=|$driver|");
        return ($driver);
}

#
# Festsetzung einer datasource aus dbsystem und dem Parameter und den Werten fr Server und Port
#
sub dbase_get_dbname {
        my $name = shift;

        return (undef) unless $$rparms{'dbsystem'};
        my $dbsystem = $$rparms{'dbsystem'};

        my $dbname = "dbi:$$rparms{'dbsystem'}:";
        if ($dbsystem eq 'Pg') {
                $dbname .= "dbname=$name";
                $dbname .= ";host=$$rparms{'server'}" if $$rparms{'server'};
                $dbname .= ";prot=$$rparms{'port'}" if $$rparms{'port'};
        } elsif ($dbsystem eq 'ODBC') {
                $dbname .= $name;
        } else {
                &dbError(&gtext('unzdriv',$dbsystem));
        }
        &dbprint(9,"dbase_get_dbname: name=|$name|, dbname=|$dbname|");
        return($dbname);
}

#
# Ermittlung des/der Datenbank DBI-Treiber
#
sub dbase_get_driver {

        my @dbSystems   = DBI->available_drivers;
        &dbError(&gtext("dbsyserror")) unless (@dbSystems);
        &dbprint(9,"dbase_get_driver: Vorhandene bzw. gewnschte DBI-Treiber: |@dbSystems|");
        return(@dbSystems);
}

#
# Ermittlung der fr den DBI-Treiber zul�sigen Data_Sources
#
sub dbase_get_sources {
        $dbsystem       = shift;
        my @dbDatas;

        my @allowed_dbs = &get_db_array('allowed_dbs');
        my @exclude_dbs = &get_db_array('exclude_dbs');

        return (undef)  unless $dbsystem;

        my @dbases      = DBI->data_sources("$dbsystem");

        # Check for allowed or excluded databases
        foreach my $dbs (@dbases) {
                my $db_name = &dbase_get_name($dbs);
                my $allow   = 0;
                my $exclude = 0;

                foreach my $aldb (@allowed_dbs) {
                        next unless $aldb;
                        if ($db_name eq $aldb) {
                                push(@dbDatas,$dbs);
                                $allow = 1;
                                last;
                        }
                }
                next if $allow;
                foreach my $excl (@exclude_dbs) {
                        next unless $excl;
                        if ($db_name eq $excl) {
                                $exclude = 1;
                                last;
                        }
                }
                push(@dbDatas,$dbs) unless $exclude;
        }

        &dbError(&gtext("dbserror",$dbsystem)) unless (@dbDatas);
        &dbprint(9,"dbase_get_sources: Vorhandene bzw. gewnschte Datenbanken = |@dbDatas|");

        return (@dbDatas);
}

#
# this routine gets the last modification date of our record
# and compares it with the date that was given from the
# caller (i.e. fetched right before the target record was
# displayed the last time) in $tmin
# if they don't match the user will get an error
# if succesfull checked the command given as parameter will be done
sub db_check_row {
        my $oid    = shift;

        if ($$rfielddesc{'tim'} && $$rfielddesc{'tmin'}{'TYPE'} =~ /^timestamp/i ) {
                my $comm   = qq{SELECT "tmin" FROM "$table" WHERE "oid"=$oid};
                my $result = &db_db_get('dbase',$comm);

                my $rhash  = &db_db_hashref($result);
                my $tmin   = $$rhash{'tmin'};
                $result->finish;

                $tmin   =~ s/ //g;
                &dbprint (7,"db_check_do_cmd: oid=$oid<BR>\ntmin=$tmin und tmin_alt=" . $$rparms{'tmin'});

                #
                # terminate dbengine.cgi if the dates don't match
                if($tmin ne $$rparms{'tmin'}) {
                                &pr_list_start("update problem");

                        &print(gtext("<H4>datsatznew</H4>\n"));

                        &pr_create_appendref(gtext("newread"));

                                &pr_db_finish;
                                return(0);
                }
        }
        return(1);
}

#
# Routine to find the childtables of a given table
#
sub db_get_childs {
        my ($table, $rchilds) =  @_;
        my $cmd = qq{SELECT * FROM relation WHERE parent='$table' AND DBaseName='$dbasen'};

        my $result = &db_db_get('dbdesc',$cmd);

        while(my $rarray = &db_db_hashref($result)) {
                my $name = $$rarray{'parentfield'};
                $$rchilds{$name}{'childcheck'} = $$rarray{'childcheck'};
                my $rcurrent               = $$rchilds{$name};

                $$rcurrent{'child'}        = $$rarray{'child'};
                $$rcurrent{'childfield'}   = $$rarray{'childfield'};
        }
        $result->finish;
}

#
# Routine to check the existence of a child-record wirth the value to change
#
sub db_check_child {
        my ($name, $content, $rchilds) = @_;
#        &dbprint_hash(7,"db_check_child: name=|$name|, content=|$content|, rchilds",$rchilds);
        return(0) unless ($$rchilds{$name} || $$rvalues{$name} || ($content && $$rvalues{$name} eq $content));

        my $nchilds = $$rchilds{$name};

        return(0) unless $$nchilds{'childcheck'};

        my $cmd     = q{SELECT count(*) AS anz FROM "} . $$nchilds{'child'} . q{" WHERE "} . $$nchilds{'childfield'} . q{" = ?};
        my @bind_values;
        push(@bind_values,$$rvalues{$name});

        my $result  = &db_db_get('dbase',$cmd,\@bind_values);
        my $rarray  = &db_db_hashref($result);
        my $anz = $$rarray{'anz'};
        &dbprint_hash(7,"db_check_child: anz=|$anz|, rarray", $rarray);
        $result->finish;

        if ( $anz != 0 && $$rvalues{$name} ne $content) {
                &dbWarning(gtext('childexist',$$rvalues{$name}, $name,$$nchilds{'child'},$content));
                return(1);
        }
        return(0);
}

#
#
# subroutine for translated texts with 2 parameters: command and value to bind
# Getting the language dependend text from the defining database
#
my @langs;
sub db_gtextb {
        my ($cmd, $field, $value) = @_;
        my @bind_values;
        my $format;

        &dbprint(8,"db_gtextb: cmd=|$cmd|, field=|$field|, bindvalue=|$value|");
        return($value) unless $dbconns{'dbdesc'}; # return old value if there is no connection to the database

        @langs = split(/,/,$$rconfigs{'langs'}) unless (@langs);
        $cmd .= " AND language = ?";
        foreach my $lang (@langs) {
                @bind_values = ("$value",$lang);
                my $stdh = &db_db_get('dbdesc',$cmd,\@bind_values);
                &dbprint(9,"db_gtextb: langs=|$$rconfigs{'langs'}|, lang=|$lang|, value=|$value|");

                my $rarray = &db_db_hashref($stdh);
                next unless $rarray;
                &dbprint_hash(9,"db_gtextb: configs{'lang'}=|$$rconfigs{'lang'}|, rarray",$rarray);
                $format = $$rarray{$field};
                $stdh->finish if $stdh;
                last if $format;
        }
        return(($format) ? $format : $value);
}


#
# Getting Fieldtypes of a table of a database
#
# global hasharrayrefs for result as Parameters
#
sub db_getFieldTypes {
        #
        # we request a list of all fieldnames, types and displayinfos for $table
        # the result then is stored in the ass. arrays %fieldtype, %fieldlength,
        # %fielddisplay, %fielddispstring, %fielddisporder, %fielddispcolumn, %fielddefault and %fieldeval

        my $rarray;
        my $tschema = "public";
        my $stable  = $table;
        if ( $table =~ /^([^\.]+)\.(.+)$/ ) {
                $tschema = $1;
                $stable  = $2;
        }
        my $stdh = $dbordh->column_info('',$tschema,$stable,'');
        while ( my $rarray = $stdh->fetchrow_hashref ) {
                &dbprint_hash(7,"db_getFieldTypes: stdh=|$stdh|, rarray",$rarray);
                my $name                    = $$rarray{'COLUMN_NAME'};
		$name                       = $1 if ( $name =~ /^\"(.+)\"$/ );
                &dbprint_hash(9,"db_getFieldTypes: name=|$name| rarray", $rarray);
#                $$rfielddesc{$name}{'TYPE'} = $$rarray{'DATA_TYPE'};
                $$rfielddesc{$name}{'TYPE'} = $$rarray{'TYPE_NAME'};
                my $rcurrent                = $$rfielddesc{$name};
                $$rcurrent{'SIZE'} = $$rarray{'COLUMN_SIZE'};
                $$rcurrent{'NOTNULL'}    = $$rarray{'attnotnull'};
                $$rcurrent{'NAME'}       = $name;

#                foreach my $key (keys %$rarray) {
#                        $$rcurrent{$key} = $$rarray{$key};
#                }
        }
        #
        # now we need a second request to get additional design information from dbinfo (if any)
        #
        my $unddispord = 1000; # maximal 1000 fields with a displayorder number!!!
        my $cmd = qq{SELECT * FROM designinfo WHERE tablename=\'$table\' AND dbasename=\'$dbasen\'};
        my $result = &db_db_get('dbdesc',$cmd);
        &dbprint(7,"getFieldTypes: aus dbdesc cmd=|$cmd|, result=|$result|");

        while ($rarray = &db_db_hashref($result)) {
                my $current     = $$rarray{'fieldname'};
                $$rfielddesc{$current}{'DISPLAY'}    = $$rarray{'displayinfo'}; # needed to define the hash for $current if the field
                                                                                # is not defined by relation-attributes (virtualfield)
                my $rcurrent    = $$rfielddesc{$current};
                $$rcurrent{'DISPLAY'}    = "" unless $$rcurrent{'DISPLAY'};
                &dbprint_hash(7,"db_getFieldTypes rarray:",$rarray);

                $$rcurrent{'DISPTABLE'}  = $$rarray{'disptable'};
                $$rcurrent{'DISPVFIELD'} = $$rarray{'dispvfield'};
                $$rcurrent{'DISPDFIELD'} = $$rarray{'dispdfield'};
                my $dispstr = $$rarray{'displaystring'};
                $dispstr    = "&nbsp;" if ($$rcurrent{'DISPLAY'} && $$rcurrent{'DISPLAY'} eq "NONE");
                $dispstr    = "" unless $dispstr;
                $dispstr    = &gtext($1) if ( $dispstr =~ /^lang\((.+)\)$/ );
                $$rcurrent{'DISPSTR'}    = $dispstr;
                $$rcurrent{'XDEFAULT'}   = $$rarray{'xdefault'};        # Ma y override the default of the tableschema
                $$rcurrent{'EVAL'}       = $$rarray{'xevaluation'};
                my $DISPCOL              = $$rarray{'displaycolumn'};
                $DISPCOL                 = 1 unless $DISPCOL;
                $$rcurrent{'ROWS'}       = $$rarray{'rowspan'};
                my $COLS                 = $$rarray{'colspan'};
                $$rcurrent{'COLS'}       = ( $COLS ? $COLS : 1 );

                #
                # make sure that the location information is correct when
                # sorted with sort and xSort (and ASCII-compared...)
                while(length($DISPCOL) < 5) {
                        $DISPCOL = "0" . $DISPCOL;
                }
                $$rcurrent{'DISPCOL'} = $DISPCOL;

                my $zwerg = $$rarray{'displayorder'};
                $zwerg = ++$unddispord unless $zwerg;
                while(length($zwerg) < 5) {
                        $zwerg = "0" . $zwerg;
                }

                my $txtcount = "000";
                $$rcurrent{'DISPORDER'} = $zwerg . "000" . $$rcurrent{'DISPCOL'};
                &dbprint_hash(9,"db_getFieldTypes current=|$current|,rcurrent",$rcurrent);
        }
        &dbprint_hash_hash(9,"db_getFieldTypes: rfielddesc",$rfielddesc);


        # for fields without a 'DISPLAY'-information search for information as referenz to a parent
        foreach my $name (keys %$rfielddesc) {
                my $rcurrent = $$rfielddesc{$name};
                unless ( $$rcurrent{'DISPLAY'} ) {
                        my $cmd = q{SELECT * FROM relation } .
                                  qq{WHERE childfield='$name' AND } .
                                  qq{child='$table' AND } .
                                  qq{DBaseName='$dbasen'};
                        my $relations   = &db_db_get('dbdesc',$cmd);
                        &dbprint(9,"db_getFieldTypes: cmd=|$cmd|, name=|$name|");
                        if ( (my $rarray        = &db_db_hashref($relations)) ) {
                                &dbprint_hash(9,"db_getFieldTypes: relations rarray",$rarray);
                                my $relvalue    = "relationpopup " . $$rarray{'parent'} . " " .
                                                  $$rarray{'parentfield'};
                                $$rcurrent{'DISPLAY'} = $relvalue;
                        }
                        $relations->finish;
                }
        }
}

#
# let's try to get additional design information (if any)
# for the proper display of the current $table and store that
# information in $tableVirtuals, $tableStart, $tableItem, $tableEnd and $tableQuery
sub db_getTableDesign {
        my $rtableDesign = shift;

        my $cmd = qq{SELECT * FROM tabledesign WHERE tablename='$table' AND DBaseName='$dbasen'};
        my $result = &db_db_get('dbdesc',$cmd);

        my $rarray = &db_db_hashref($result);


        $$rtableDesign{'current'}         = $$rarray{'tablename'};
        $$rtableDesign{'tableVirtuals'}   = $$rarray{'needsvirtuals'};
        my $tabf = $$rarray{'tablefields'};
        $tabf =~ tr/\n\r/  / if($tabf);
        $$rtableDesign{'tableFields'}     = $tabf;
        $$rtableDesign{'tableStart'}      = $$rarray{'xtablestart'};
        $$rtableDesign{'tableItem'}       = $$rarray{'xtableitem'};
        $$rtableDesign{'tableEnd'}        = $$rarray{'xtableend'};
        $$rtableDesign{'tableQuery'}      = $$rarray{'xxquerystring'};
        $$rtableDesign{'tableModifyable'} = $$rarray{'modifyablelist'};

        $result->finish;
}

#
# Fill Values with tuple-data and data from virtuel fields
#
sub db_fillValues {
        #
        # if there is a valid $oid try to fetch that record from $table
        # and copy it's contents into %values
        &dbprint_hash_hash(9,"db_fillValues: fielddesc",$rfielddesc);
        if (my $oid = $$rparms{'oid'}) {
                &db_get_row($table, $oid);
                &dbprint_hash(9,"db_fillValues: oid=|$oid| values",$rvalues);

                #
                # then add the contents of any defined virtual fields
                &db_getVirtualFields;

                #
                # now we try to fetch relation records for $table
                # and store them in $relations
                my $tuples = 0;
                foreach my $wer ('parent', 'child') {
                        my $cmd       = qq{SELECT * FROM relation WHERE $wer='$table' AND DBaseName='$dbasen'};
                        my $relations = &db_db_get('dbdesc',$cmd);

                        while ( my $rarray  = &db_db_hashref($relations) ) {
                                foreach my $chkey (keys %$rarray) {
                                        $$rrelations{$wer}[$tuples]{$chkey} = $$rarray{$chkey};
                                }
                                $tuples++;
                                &dbprint_hash(7,"db_fillvalues: wer=|$wer|, tuples=|$tuples|, rrelart",$$rrelations{$wer}[$tuples]);
                        }
                        $$rparms{$wer} = $tuples;
                }
        #
        # if there's no oid we'll have to display an empty dialog
        # and therefore reset all our values to ""
        } else {
                foreach my $name (keys %$rfielddesc) {
                        my $rcurrent    = $$rfielddesc{$name};
                        if($$rcurrent{'XDEFAULT'}) {
                                #
                                # if there's a Perl subroutine specified to
                                # set the default value of this field, ask it...
                                $$rvalues{$name} = eval ($$rcurrent{'XDEFAULT'});
                                $$rvalues{$name} = $@ if $@;
                                &dbprint(9,"db_fillValues: name=|$name|, xdefault=|$$rcurrent{'XDEFAULT'}|," .
                                           " value=|$$rvalues{$name}|");
#                        } elsif ($$rcurrent{'DEFAULT'}) {
#                                #
#                                # otherwise get the defaultvalue from the tableschema
#                                $$rvalues{$name} = $$rcurrent{'DEFAULT'};
                        } elsif ($$rcurrent{'TYPE'} =~ /^bool/ ) {
                                $$rvalues{$name} = "u";
                        } else {
                                $$rvalues{$name} = "";
                        }
                }
        }
        &dbprint_hash(7,"db_fillValues: rvalues", $rvalues);
        &dbprint_hash_hash(9,"db_fillValues: fielddesc",$rfielddesc);
}

#
# This routine scans the table virtual for virtual fields that belong to $table
# and evaluates the xequation fields of their records to store their values
# into the %values array.
# if the's any error during this request we simply ignore it
#
sub db_getVirtualFields {

        my $comm    = qq{SELECT * FROM virtual WHERE tablename='$table' }  .
                                            qq{AND DBaseName='$dbasen' ORDER BY level};
        my $vfields = &db_db_get('dbdesc',$comm);

        while(my $rarray = &db_db_hashref($vfields)) {
                my $erg;
                my $current = $$rarray{'fieldname'};

                next unless ($$rfielddesc{$current});

                $$rfielddesc{$current}{'TYPE'} = "VIRTUAL " . $$rarray{'fieldtype'};
                $$rvalues{$current} = $erg     = eval ($$rarray{'xequation'});
                &dbprint(7,"db_getVirtualFields: \$@=|$@|, table=|$table|, current=|$current|, erg=|$erg|");
                #
                # if there's an error during the evaluation we set our
                # virtual fields  contents to the error message
                if ($@) {
                        $$rvalues{$current} = "<H4><FONT COLOR=red><P>Error in virtual field $current:<BR>\n" . $@ . "</FONT></H4>";
                }
        }
}

#
# this little tool routine removes all spaces from
# the end of a given string
#
sub xChop {
        my $var = shift;
        my $i;
        return $var unless $var;
        for ( $i = length($var) - 1; $i >= 0; $i-- ) {
                last if (substr($var,$i,1) ne " ");
        }
        return(substr($var,0,$i+1));
}

#
# Function to change a value dependuing on Attribute-type to a value accepted by SQL
# The result is a SQL-WHERE-clause
sub db_value2SQL {
        my ($rsuche, $rbind_values, $name, $rquer_wert) = @_;
        my $erg         = 1;              # wenn true, dann 'AND' anh�gen
        my $dispname    = $$rfielddesc{$name}{'DISPLAY'};
        $dispname       = "" unless $dispname;
        my $foptype     = $$rfielddesc{$name}{'TYPE'};
        my $content;
        my $quer_wert   = $content = &pr_get_relvalues($name,$rquer_wert);

        $$rsuche .= " AND " if (db_ValueToSQL($rsuche, $rbind_values, $name, $quer_wert, $dispname, $foptype, 1));
        return ($content);
}

#
# Function making the possible changes but also for external uses
#
sub db_ValueToSQL {
        my ($rsuche, $rbind_values, $name, $quer_wert, $dispname, $foptype, $erg) = @_;

        #                  Festlegung f�r welche Datentypen welche Operatoren zul�ssig sind:
        #                  1 = 2**0, char's
        #                  2 = 2**1, date
        #                  4 = 2**2, bool
        #                  8 = 2**3, alle brigen
        my $vgl = "";

        my $foptypen    = "";
        my $fopnumber   = 0;

        ($vgl,$quer_wert,$foptypen,$fopnumber) = &pr_get_operator($foptype,$quer_wert);

        #
        # je nach Datentyp wird unser Suchbegriff nun gelegentlich umgeformt
        #         Datentyp = Zeichenkette (char - Types und text)
#        &dbprint(7,"db_ValueToSQL: fopnumber=|$fopnumber|, quer_wert=|$quer_wert|, type=|$foptype|, vgl=|$vgl|" .
#                   "\nname=|$name|, dispname=|$dispname|, erg=|$erg|");
        if($fopnumber == 1 ) {
                $erg = &pr_char_suchw($name,$quer_wert,$vgl,$foptype,$dispname,$rsuche,$rbind_values);
        #
        # bei Dates suchen wir �erstimmungen in Anfhrungszeichen
        # Date-Types
        } elsif($fopnumber == 2) {
                &pr_date_suchw($name,$quer_wert,$vgl,$rsuche,$rbind_values);
        #
        # bei Booleans suchen wir �ereinstimmungen in Anfhrungszeichen
        # Nur zwei Werte zul�sig, der Wert 'u' fr undefiniert darf nicht weitergegeben werden
        } elsif($fopnumber == 4) {
                $erg = &pr_bool_suchw($name,$quer_wert,$vgl,$rsuche,$rbind_values);
        #
        # bei Zahlen ist das nicht erforderlich
        } else {
                &pr_number_suchw($name,$quer_wert,$vgl,$foptype,$rsuche,$rbind_values);
        }
#        &dbprint(7,"db_ValueToSQL: vgl=|$vgl|, rsuche=|$$rsuche|, quer_wert=|$quer_wert|");
        return($erg);
}

#
# Function to change a value depending on Attribute-type to a value for output
# The output depends on the given Language in configs. It's essential
# for numerical values
sub db_SQL2value {
        my ($name, $inpval) = @_;

        &dbprint_hash(9,"db_SQL2value: name=|$name|, rfielddesc", $$rfielddesc{$name});
        my $ftype  = $$rfielddesc{$name}{'TYPE'};
        $ftype  = "" unless $ftype;
        if ($$rconfigs{'lang'}  =~ /de/) {
                &dbprint(9,"db_SQL2value: configs{lang}=|$$rconfigs{'lang'}|, ftype=|$ftype|, name=|$name|");
                if ( $ftype =~ /float/i || $ftype =~ /numeric/i ) {
                        if($inpval =~ /^(.+)\.(\d+)$/ ) {
                                $inpval = "$1,$2";
                        }
                }
        }
        return($inpval);
}

#
# Subroutine to check input dates for playing good with DateStyle
#
sub db_inp_date {
        my $datum = shift;
        my ($jahr, $monat, $tag);
        my $istyle = &db_show_variable('DateStyle');
        &dbprint(7,"db_inp_date: datum=|$datum|, istyle=|$istyle|, lang=|" . $$rconfigs{'lang'} . "|");
        if ($datum =~ /^((Q|H)?\d+)\/(\d+)$/) {   # Quartale, Halbjahre oder Monate
                $jahr = $3; $monat = $1; $tag = '';
        } elsif ($datum =~ /^\[(\d\d?(\.|\/|-)\d\d?(\.|\/|-)\d{1,4})\s*-\s*(\d\d?(\.|\/|-)\d\d?(\.|\/|-)\d{1,4})\]$/) {
                &dbError(&gtext('illeg_date',$datum));
        } elsif ($datum =~ /^(\d{1,4})$/) {  # Angabe nur von jahr
                $jahr = $1; $monat = ''; $tag = '';
        } elsif ($$rconfigs{'lang'} !~ /us/ ) {
                if ($datum =~ /^(\d\d?).(\d\d?).(\d{1,4})$/ ) { # Angabe von Tag, Monat und Jahr
                        ($tag,$monat,$jahr) = ($1, $2, $3);
                } elsif ($datum =~ /^(\d\d?)\.(\d\d?)\.$/ ){ # Angabe nur von Tag, Monat
                        return($1, $2, '');
                } else {
                        &dbError(&gtext('illeg_date',"datum(DMY): $datum"));
                }
        } else  {
                if ($datum =~ /(d\d?).(\d\d?).(\d{1,4})/) {
                        ($monat,$tag,$jahr) = ($1, $2, $3);
                } elsif ($datum =~ /^(\d\d?)[^\d](\d\d?)[^\d]$/ ){ # Angabe nur von Tag, Monat
                        return($1, $2, '');
                } else {
                        &dbError(&gtext('illeg_date',"datum(MDY): $datum"));
                }
        }
        if ($jahr < 45) {
                $jahr += 2000;
        } elsif ($jahr < 1000) {
                $jahr += 1900;
        }
        return($tag,$monat,$jahr);
}

#
# Subroutine to define an exact date for the given datestyle
# Input is of the Form dd-mm-yyyy
#
sub db_bind_date {
        my @datum = split(/-/,shift);
        my $istyle = &db_show_variable('DateStyle');
        my $datum;

        if ($istyle =~ /ISO/) {
                $datum = "$datum[2]-$datum[1]-$datum[0]";
        } elsif ($istyle =~ /German/) {
                $datum = "$datum[0].$datum[1].$datum[2]";
        } elsif ($istyle =~ /(SQL|US)/) {
                if ($istyle =~ /(NonEurop|US)/ || $istyle eq 'SQL') {
                        $datum = "$datum[1]/$datum[0]/$datum[2]";
                } else {
                        $datum = "$datum[0]/$datum[1]/$datum[2]";
                }
        } elsif ($istyle =~ /Postgres/ ) {
                if ($istyle =~ /(NonEurop|US)/) {
                        $datum = "$datum[1]-$datum[0]-$datum[2]";
                } else {
                        $datum = "$datum[0]-$datum[1]-$datum[2]";
                }
        }
        dbprint(7,"db_bind_date: datum=|$datum|, istyle=|$istyle|");
        return($datum);
}

#
# this subroutine expects the name of a perl routine stored in the
# dbase table equation as first argument, followed by the routines
# arguments.
# it then tries to get the routine from our database and to
# evaluate it.
# if there occurs any error $@ is set to the error message and
# the routine 'dies'. Otherwise the routine returns whatever value
# the last expression of our routine has.
sub db_callSub {
        my ($name, @args) = @_;
        &dbprint(9,"db_callsub: name=|$name|, args=|" . join("|",@args) ."|");
        my $ret;
        my $rarray;

        my $cmd    = qq{SELECT content FROM equation WHERE eqname = \'$name\' AND dbasename = \'$dbasen\'};
        my $result = &db_db_get('dbdesc', $cmd);

        unless ( $rarray = &db_db_hashref($result) ) {
                $result->finish;
                die(&gtext('dbsuberr', $name));
        } else {
                my $subr = $$rarray{'content'};
                $result->finish;
                @_   = @args;
                $ret = eval $subr;
                &dbError(&gtext('dbsubouterr', $name) . "<PRE>" . $@ . "</PRE>")  if $@;
        }
        return($ret);
}

#
# Subroutine to load all possible timezones
#
sub db_get_timezones {
        my $rarray = shift;
        foreach my $zone (`ls /usr/share/zoneinfo`) {
                chomp($zone);
                next unless ($zone =~ /^[A-Z]/);
                if ( -d "/usr/share/zoneinfo/$zone") {
                        foreach my $subzone (`ls /usr/share/zoneinfo/$zone`) {
                                push(@$rarray,"$zone/$subzone");
                        }
                } else {
                        push(@$rarray,$zone);
                }
        }
}

#
# Loading one Parameter of the Hashfield configs
#
sub db_get_configs {
        my $conf_wen = shift;
        return($$rconfigs{$conf_wen});
}


1;
