#########################
#
# dbengine_userdefine.pl
#
# Modul zu dbengine.cgi: Verwaltung und Nutzung von Datenbanken ber
# Browserschnittstelle.
#
# Version 2.3.0
# <c> 2003, 2004 Detlef Duerr
# get closer Information from email Detlef.Duerr@gmx.de
#
# Local Library for executing special standardactions und starting the
# executing of all other actions
#
########################

#use strict;

my $rparms;
my $rconfigs;
my $qu;

sub userdefine_init {
        ($rconfigs,$rparms,$qu) = @_;
        &dbprint(7,"userdefine_init: rparms=|$rparms|, rconfigs=|$rconfigs|, qu=|$qu|");
}

#
# Destroy old kennz-Files
#
sub dest_old_kennz_files {
        my $dir = $$rconfigs{'usrSaveDir'};
        my $dtime = time - ($$rconfigs{'otime'}?$$rconfigs{'otime'}:86400);
        &dbprint(7,"dest_old_kennz_files: dir=|$dir|");
        if (opendir(KENNZ,$dir)) {
                my @files = readdir(KENNZ);
                foreach my $file (@files) {
                        &dbprint (12,"dest_old_files: dir=|$dir|, file=|$file|");
                        next if ($file =~ /^\.+$/);
                        if (-r "$dir/$file") {
                                my ($otime,$rest) = split(/-/,$file);
                                &dbprint (12,"dest_old_files: otime=|$otime|, dtime=|$dtime|");
                                if ($otime <= $dtime) {
                                        unlink "$dir/$file";
                                }
                        }
                }
                closedir(KENNZ);
        }
}

#
# Abfrage von User und Passwort fr Datenbank, sofern kein User vorhanden
# Dann Ende der Bearbeitung.
#
sub userget {

        &pr_form_start(&gtext('utitle'));
        &pr_form_pw;
        $$rparms{'mode'} = 'userdefine';
        &pr_form_stop('   OK   ','  RESET ');
        &pr_db_stop;
        return;
}

#
# �ernahme von User und Passwort aus den Daten, sofern vorhanden
# Erzeugen der abzuspeichernden user-information. Ggbflls. Datenbank anfordern.
#
sub userdefine {
        local *parms   = $rparms;
        local *configs = $rconfigs;

        my $user    = $parms{'user'};
        my $pwd     = $parms{'pwd'};

        &dbError(&gtext("usermust")) unless ($user && $pwd);
        &dbError(&gtext("usererror",$user)) if ($user =~ /[^A-Za-z_]/);

        # old files (older as configs{'otime'} or default (1 day) has to be destroyed
        # and setting of the user-specific dates in an special file
        &dest_old_kennz_files;

        # Definition des 'user' mit 'passwort' und Maschine und eindeutiger Prozesskennzeichnung.
        # Die eindeutige Kennzeichnung fr den user wird als hidden-wert weitergegeben
        # und sp�er wieder ausgelesen.
        &dbprint_hash(7,"userdefine: parms",\%parms);
        unless ($parms{'dbase'}) {
                &dbError(&gtext('dbsyserror')) unless ($parms{'dbsystem'});
                &dbprint(7,"userdefine: mode vor Formstart=|$parms{'mode'}|, kennz=|$parms{'kennz'}|");

                my @dbdats = &dbase_get_sources($parms{'dbsystem'});

                &pr_form_start(&gtext("dbase"));
                &pr_form_pw;
                &pr_form_stop('   OK   ','  RESET ');
                &pr_db_stop;
                &dbprint(7,"userdefine: mode nach Formausgabe=|$parms{'mode'}|, kennz=|$parms{'kennz'}|");
                $parms{'kennz'} = &set_user_file;
                return;
        } else {
                # Datenbank ist vorhanden => Start des eigentlichen Systems
                # find the search-path for schemas
                if (my $pathconf = $configs{'search_pathes'} ) {
                        unless ( defined $parms{'__schema_group_d'}) {  # The user must define his SCHEMA-searchpath
                                # lets open the databases: the defining and the ordered
                                &db_db_conn(1);
                                &dbprint(7,"userdefine: pathcof=|$pathconf|, user=|$user|");
                                my @pathlist = &db_db_schemas($user,$pathconf);
                                &dbprint_array(7,"userdefine: schemas",\@pathlist);
                                &dbprint_array(7,"userdefine: pathlist=|$pathconf|, searchlist",\@pathlist);
                                &dbError(&gtext('no_found_schem',$parms{'dbasen'},$user)) unless (@pathlist);
                                if ($#pathlist) {
                                        print qq{<body bgcolor="$configs{'bgcol'}" -base="true">\n};
                                        &pr_form_start(&gtext('stitle',$parms{'dbasen'}));
                                        print $qu->hidden('__schema_group_d', 1);
                                        print "<H4><CENTER>" . &gtext('def_schema') . "</CENTER></H4>" .
                                                "<CENTER><TABLE Border=0 cellpadding=10>\n<TR><TD><H4>&nbsp;</H4><TD> </TD></TR>" .
                                                "<TR><TD><H4>" . &gtext('pschemas') . ": </H4></TD><TD><H4>\n";
                                        print $qu->checkbox_group(-name=>'__schema_group',
                                                                  -values=>\@pathlist,
                                                                  -linebreak=>'true') . "</H4></TD></TR>\n";
                                        $parms{'mode'} = 'userdefine';
                                        $parms{'kennz'} = &set_user_file;   # Storing the user-parms given until the schemas are given
                                        &pr_form_stop('   OK   ','  RESET ');
                                        &pr_db_stop;
                                        &db_db_disc;     # Ending the database-connections
                                        return;
                                } else {
                                        $parms{'schemas'} = $pathlist[0];
                                }
                        } else {
                                if ( my @schema_group = $qu->param('__schema_group') ) { # define searchpath only if schemanames given
                                        $parms{'schemas'} = join(',',@schema_group);
                                        &dbprint(7,"db_db_tables: schemas=|$parms{'schemas'}|");
                                }
                        }
                }
                my $kennz = $parms{'kennz'} = &set_user_file;  # Storing all user_parms including schemas
                print <<EOM
<FRAMESET Cols=\"185,*\">
<FRAME Name=menu SRC=dbengine.cgi?dbase=$parms{'dbase'}&mode=menu&user=$user&kennz=$kennz Scrolling=Auto>
<FRAME Name=body SRC=dbengine.cgi?dbase=$parms{'dbase'}&mode=empty&user=$user&kennz=$kennz Scrolling=Auto>
</FRAMESET>
</HTML>
EOM
;

                &dbprint_hash(7,"db_db_tables: parms",\%parms);
                dbprint(7,"userdefine: dbase=|$parms{'dbase'}|, mode=|$parms{'mode'}|, kennz=|$parms{'kennz'}|");
                &pr_db_finish;
        }
}

1;
